# This file was auto-generated by Fern from our API Definition.

import typing
from ..core.client_wrapper import SyncClientWrapper
from ..requests.booking import BookingParams
from ..core.request_options import RequestOptions
from ..core.http_response import HttpResponse
from ..types.create_booking_response import CreateBookingResponse
from ..core.serialization import convert_and_respect_annotation_metadata
from ..core.unchecked_base_model import construct_type
from json.decoder import JSONDecodeError
from ..core.api_error import ApiError
from ..requests.search_availability_query import SearchAvailabilityQueryParams
from ..types.search_availability_response import SearchAvailabilityResponse
from ..types.bulk_retrieve_bookings_response import BulkRetrieveBookingsResponse
from ..types.get_business_booking_profile_response import GetBusinessBookingProfileResponse
from ..types.retrieve_location_booking_profile_response import RetrieveLocationBookingProfileResponse
from ..core.jsonable_encoder import jsonable_encoder
from ..types.bulk_retrieve_team_member_booking_profiles_response import BulkRetrieveTeamMemberBookingProfilesResponse
from ..types.get_booking_response import GetBookingResponse
from ..types.update_booking_response import UpdateBookingResponse
from ..types.cancel_booking_response import CancelBookingResponse
from ..core.client_wrapper import AsyncClientWrapper
from ..core.http_response import AsyncHttpResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawBookingsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def create(
        self,
        *,
        booking: BookingParams,
        idempotency_key: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[CreateBookingResponse]:
        """
        Creates a booking.

        The required input must include the following:
        - `Booking.location_id`
        - `Booking.start_at`
        - `Booking.AppointmentSegment.team_member_id`
        - `Booking.AppointmentSegment.service_variation_id`
        - `Booking.AppointmentSegment.service_variation_version`

        To call this endpoint with buyer-level permissions, set `APPOINTMENTS_WRITE` for the OAuth scope.
        To call this endpoint with seller-level permissions, set `APPOINTMENTS_ALL_WRITE` and `APPOINTMENTS_WRITE` for the OAuth scope.

        For calls to this endpoint with seller-level permissions to succeed, the seller must have subscribed to *Appointments Plus*
        or *Appointments Premium*.

        Parameters
        ----------
        booking : BookingParams
            The details of the booking to be created.

        idempotency_key : typing.Optional[str]
            A unique key to make this request an idempotent operation.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[CreateBookingResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            "v2/bookings",
            method="POST",
            json={
                "idempotency_key": idempotency_key,
                "booking": convert_and_respect_annotation_metadata(
                    object_=booking, annotation=BookingParams, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreateBookingResponse,
                    construct_type(
                        type_=CreateBookingResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def search_availability(
        self, *, query: SearchAvailabilityQueryParams, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[SearchAvailabilityResponse]:
        """
        Searches for availabilities for booking.

        To call this endpoint with buyer-level permissions, set `APPOINTMENTS_READ` for the OAuth scope.
        To call this endpoint with seller-level permissions, set `APPOINTMENTS_ALL_READ` and `APPOINTMENTS_READ` for the OAuth scope.

        Parameters
        ----------
        query : SearchAvailabilityQueryParams
            Query conditions used to filter buyer-accessible booking availabilities.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[SearchAvailabilityResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            "v2/bookings/availability/search",
            method="POST",
            json={
                "query": convert_and_respect_annotation_metadata(
                    object_=query, annotation=SearchAvailabilityQueryParams, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    SearchAvailabilityResponse,
                    construct_type(
                        type_=SearchAvailabilityResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def bulk_retrieve_bookings(
        self, *, booking_ids: typing.Sequence[str], request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[BulkRetrieveBookingsResponse]:
        """
        Bulk-Retrieves a list of bookings by booking IDs.

        To call this endpoint with buyer-level permissions, set `APPOINTMENTS_READ` for the OAuth scope.
        To call this endpoint with seller-level permissions, set `APPOINTMENTS_ALL_READ` and `APPOINTMENTS_READ` for the OAuth scope.

        Parameters
        ----------
        booking_ids : typing.Sequence[str]
            A non-empty list of [Booking](entity:Booking) IDs specifying bookings to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[BulkRetrieveBookingsResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            "v2/bookings/bulk-retrieve",
            method="POST",
            json={
                "booking_ids": booking_ids,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    BulkRetrieveBookingsResponse,
                    construct_type(
                        type_=BulkRetrieveBookingsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get_business_profile(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[GetBusinessBookingProfileResponse]:
        """
        Retrieves a seller's booking profile.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[GetBusinessBookingProfileResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            "v2/bookings/business-booking-profile",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetBusinessBookingProfileResponse,
                    construct_type(
                        type_=GetBusinessBookingProfileResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def retrieve_location_booking_profile(
        self, location_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[RetrieveLocationBookingProfileResponse]:
        """
        Retrieves a seller's location booking profile.

        Parameters
        ----------
        location_id : str
            The ID of the location to retrieve the booking profile.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[RetrieveLocationBookingProfileResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/bookings/location-booking-profiles/{jsonable_encoder(location_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    RetrieveLocationBookingProfileResponse,
                    construct_type(
                        type_=RetrieveLocationBookingProfileResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def bulk_retrieve_team_member_booking_profiles(
        self, *, team_member_ids: typing.Sequence[str], request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[BulkRetrieveTeamMemberBookingProfilesResponse]:
        """
        Retrieves one or more team members' booking profiles.

        Parameters
        ----------
        team_member_ids : typing.Sequence[str]
            A non-empty list of IDs of team members whose booking profiles you want to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[BulkRetrieveTeamMemberBookingProfilesResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            "v2/bookings/team-member-booking-profiles/bulk-retrieve",
            method="POST",
            json={
                "team_member_ids": team_member_ids,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    BulkRetrieveTeamMemberBookingProfilesResponse,
                    construct_type(
                        type_=BulkRetrieveTeamMemberBookingProfilesResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get(
        self, booking_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[GetBookingResponse]:
        """
        Retrieves a booking.

        To call this endpoint with buyer-level permissions, set `APPOINTMENTS_READ` for the OAuth scope.
        To call this endpoint with seller-level permissions, set `APPOINTMENTS_ALL_READ` and `APPOINTMENTS_READ` for the OAuth scope.

        Parameters
        ----------
        booking_id : str
            The ID of the [Booking](entity:Booking) object representing the to-be-retrieved booking.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[GetBookingResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/bookings/{jsonable_encoder(booking_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetBookingResponse,
                    construct_type(
                        type_=GetBookingResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def update(
        self,
        booking_id: str,
        *,
        booking: BookingParams,
        idempotency_key: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[UpdateBookingResponse]:
        """
        Updates a booking.

        To call this endpoint with buyer-level permissions, set `APPOINTMENTS_WRITE` for the OAuth scope.
        To call this endpoint with seller-level permissions, set `APPOINTMENTS_ALL_WRITE` and `APPOINTMENTS_WRITE` for the OAuth scope.

        For calls to this endpoint with seller-level permissions to succeed, the seller must have subscribed to *Appointments Plus*
        or *Appointments Premium*.

        Parameters
        ----------
        booking_id : str
            The ID of the [Booking](entity:Booking) object representing the to-be-updated booking.

        booking : BookingParams
            The booking to be updated. Individual attributes explicitly specified here override the corresponding values of the existing booking.

        idempotency_key : typing.Optional[str]
            A unique key to make this request an idempotent operation.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[UpdateBookingResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/bookings/{jsonable_encoder(booking_id)}",
            method="PUT",
            json={
                "idempotency_key": idempotency_key,
                "booking": convert_and_respect_annotation_metadata(
                    object_=booking, annotation=BookingParams, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpdateBookingResponse,
                    construct_type(
                        type_=UpdateBookingResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def cancel(
        self,
        booking_id: str,
        *,
        idempotency_key: typing.Optional[str] = OMIT,
        booking_version: typing.Optional[int] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[CancelBookingResponse]:
        """
        Cancels an existing booking.

        To call this endpoint with buyer-level permissions, set `APPOINTMENTS_WRITE` for the OAuth scope.
        To call this endpoint with seller-level permissions, set `APPOINTMENTS_ALL_WRITE` and `APPOINTMENTS_WRITE` for the OAuth scope.

        For calls to this endpoint with seller-level permissions to succeed, the seller must have subscribed to *Appointments Plus*
        or *Appointments Premium*.

        Parameters
        ----------
        booking_id : str
            The ID of the [Booking](entity:Booking) object representing the to-be-cancelled booking.

        idempotency_key : typing.Optional[str]
            A unique key to make this request an idempotent operation.

        booking_version : typing.Optional[int]
            The revision number for the booking used for optimistic concurrency.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[CancelBookingResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/bookings/{jsonable_encoder(booking_id)}/cancel",
            method="POST",
            json={
                "idempotency_key": idempotency_key,
                "booking_version": booking_version,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CancelBookingResponse,
                    construct_type(
                        type_=CancelBookingResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncRawBookingsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def create(
        self,
        *,
        booking: BookingParams,
        idempotency_key: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[CreateBookingResponse]:
        """
        Creates a booking.

        The required input must include the following:
        - `Booking.location_id`
        - `Booking.start_at`
        - `Booking.AppointmentSegment.team_member_id`
        - `Booking.AppointmentSegment.service_variation_id`
        - `Booking.AppointmentSegment.service_variation_version`

        To call this endpoint with buyer-level permissions, set `APPOINTMENTS_WRITE` for the OAuth scope.
        To call this endpoint with seller-level permissions, set `APPOINTMENTS_ALL_WRITE` and `APPOINTMENTS_WRITE` for the OAuth scope.

        For calls to this endpoint with seller-level permissions to succeed, the seller must have subscribed to *Appointments Plus*
        or *Appointments Premium*.

        Parameters
        ----------
        booking : BookingParams
            The details of the booking to be created.

        idempotency_key : typing.Optional[str]
            A unique key to make this request an idempotent operation.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[CreateBookingResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v2/bookings",
            method="POST",
            json={
                "idempotency_key": idempotency_key,
                "booking": convert_and_respect_annotation_metadata(
                    object_=booking, annotation=BookingParams, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreateBookingResponse,
                    construct_type(
                        type_=CreateBookingResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def search_availability(
        self, *, query: SearchAvailabilityQueryParams, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[SearchAvailabilityResponse]:
        """
        Searches for availabilities for booking.

        To call this endpoint with buyer-level permissions, set `APPOINTMENTS_READ` for the OAuth scope.
        To call this endpoint with seller-level permissions, set `APPOINTMENTS_ALL_READ` and `APPOINTMENTS_READ` for the OAuth scope.

        Parameters
        ----------
        query : SearchAvailabilityQueryParams
            Query conditions used to filter buyer-accessible booking availabilities.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[SearchAvailabilityResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v2/bookings/availability/search",
            method="POST",
            json={
                "query": convert_and_respect_annotation_metadata(
                    object_=query, annotation=SearchAvailabilityQueryParams, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    SearchAvailabilityResponse,
                    construct_type(
                        type_=SearchAvailabilityResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def bulk_retrieve_bookings(
        self, *, booking_ids: typing.Sequence[str], request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[BulkRetrieveBookingsResponse]:
        """
        Bulk-Retrieves a list of bookings by booking IDs.

        To call this endpoint with buyer-level permissions, set `APPOINTMENTS_READ` for the OAuth scope.
        To call this endpoint with seller-level permissions, set `APPOINTMENTS_ALL_READ` and `APPOINTMENTS_READ` for the OAuth scope.

        Parameters
        ----------
        booking_ids : typing.Sequence[str]
            A non-empty list of [Booking](entity:Booking) IDs specifying bookings to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[BulkRetrieveBookingsResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v2/bookings/bulk-retrieve",
            method="POST",
            json={
                "booking_ids": booking_ids,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    BulkRetrieveBookingsResponse,
                    construct_type(
                        type_=BulkRetrieveBookingsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get_business_profile(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[GetBusinessBookingProfileResponse]:
        """
        Retrieves a seller's booking profile.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[GetBusinessBookingProfileResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v2/bookings/business-booking-profile",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetBusinessBookingProfileResponse,
                    construct_type(
                        type_=GetBusinessBookingProfileResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def retrieve_location_booking_profile(
        self, location_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[RetrieveLocationBookingProfileResponse]:
        """
        Retrieves a seller's location booking profile.

        Parameters
        ----------
        location_id : str
            The ID of the location to retrieve the booking profile.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[RetrieveLocationBookingProfileResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/bookings/location-booking-profiles/{jsonable_encoder(location_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    RetrieveLocationBookingProfileResponse,
                    construct_type(
                        type_=RetrieveLocationBookingProfileResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def bulk_retrieve_team_member_booking_profiles(
        self, *, team_member_ids: typing.Sequence[str], request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[BulkRetrieveTeamMemberBookingProfilesResponse]:
        """
        Retrieves one or more team members' booking profiles.

        Parameters
        ----------
        team_member_ids : typing.Sequence[str]
            A non-empty list of IDs of team members whose booking profiles you want to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[BulkRetrieveTeamMemberBookingProfilesResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v2/bookings/team-member-booking-profiles/bulk-retrieve",
            method="POST",
            json={
                "team_member_ids": team_member_ids,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    BulkRetrieveTeamMemberBookingProfilesResponse,
                    construct_type(
                        type_=BulkRetrieveTeamMemberBookingProfilesResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get(
        self, booking_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[GetBookingResponse]:
        """
        Retrieves a booking.

        To call this endpoint with buyer-level permissions, set `APPOINTMENTS_READ` for the OAuth scope.
        To call this endpoint with seller-level permissions, set `APPOINTMENTS_ALL_READ` and `APPOINTMENTS_READ` for the OAuth scope.

        Parameters
        ----------
        booking_id : str
            The ID of the [Booking](entity:Booking) object representing the to-be-retrieved booking.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[GetBookingResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/bookings/{jsonable_encoder(booking_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetBookingResponse,
                    construct_type(
                        type_=GetBookingResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def update(
        self,
        booking_id: str,
        *,
        booking: BookingParams,
        idempotency_key: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[UpdateBookingResponse]:
        """
        Updates a booking.

        To call this endpoint with buyer-level permissions, set `APPOINTMENTS_WRITE` for the OAuth scope.
        To call this endpoint with seller-level permissions, set `APPOINTMENTS_ALL_WRITE` and `APPOINTMENTS_WRITE` for the OAuth scope.

        For calls to this endpoint with seller-level permissions to succeed, the seller must have subscribed to *Appointments Plus*
        or *Appointments Premium*.

        Parameters
        ----------
        booking_id : str
            The ID of the [Booking](entity:Booking) object representing the to-be-updated booking.

        booking : BookingParams
            The booking to be updated. Individual attributes explicitly specified here override the corresponding values of the existing booking.

        idempotency_key : typing.Optional[str]
            A unique key to make this request an idempotent operation.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[UpdateBookingResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/bookings/{jsonable_encoder(booking_id)}",
            method="PUT",
            json={
                "idempotency_key": idempotency_key,
                "booking": convert_and_respect_annotation_metadata(
                    object_=booking, annotation=BookingParams, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpdateBookingResponse,
                    construct_type(
                        type_=UpdateBookingResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def cancel(
        self,
        booking_id: str,
        *,
        idempotency_key: typing.Optional[str] = OMIT,
        booking_version: typing.Optional[int] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[CancelBookingResponse]:
        """
        Cancels an existing booking.

        To call this endpoint with buyer-level permissions, set `APPOINTMENTS_WRITE` for the OAuth scope.
        To call this endpoint with seller-level permissions, set `APPOINTMENTS_ALL_WRITE` and `APPOINTMENTS_WRITE` for the OAuth scope.

        For calls to this endpoint with seller-level permissions to succeed, the seller must have subscribed to *Appointments Plus*
        or *Appointments Premium*.

        Parameters
        ----------
        booking_id : str
            The ID of the [Booking](entity:Booking) object representing the to-be-cancelled booking.

        idempotency_key : typing.Optional[str]
            A unique key to make this request an idempotent operation.

        booking_version : typing.Optional[int]
            The revision number for the booking used for optimistic concurrency.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[CancelBookingResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/bookings/{jsonable_encoder(booking_id)}/cancel",
            method="POST",
            json={
                "idempotency_key": idempotency_key,
                "booking_version": booking_version,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CancelBookingResponse,
                    construct_type(
                        type_=CancelBookingResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
import typing


class LoyaltyPromotionIncentivePointsAdditionData(UncheckedBaseModel):
    """
    Represents the metadata for a `POINTS_ADDITION` type of [loyalty promotion incentive](entity:LoyaltyPromotionIncentive).
    """

    points_addition: int = pydantic.Field()
    """
    The number of additional points to earn each time the promotion is triggered. For example,
    suppose a purchase qualifies for 5 points from the base loyalty program. If the purchase also
    qualifies for a `POINTS_ADDITION` promotion incentive with a `points_addition` of 3, the buyer
    earns a total of 8 points (5 program points + 3 promotion points = 8 points).
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class CatalogProductSet(UncheckedBaseModel):
    """
    Represents a collection of catalog objects for the purpose of applying a
    `PricingRule`. Including a catalog object will include all of its subtypes.
    For example, including a category in a product set will include all of its
    items and associated item variations in the product set. Including an item in
    a product set will also include its item variations.
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    User-defined name for the product set. For example, "Clearance Items"
    or "Winter Sale Items".
    """

    product_ids_any: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
     Unique IDs for any `CatalogObject` included in this product set. Any
    number of these catalog objects can be in an order for a pricing rule to apply.
    
    This can be used with `product_ids_all` in a parent `CatalogProductSet` to
    match groups of products for a bulk discount, such as a discount for an
    entree and side combo.
    
    Only one of `product_ids_all`, `product_ids_any`, or `all_products` can be set.
    
    Max: 500 catalog object IDs.
    """

    product_ids_all: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    Unique IDs for any `CatalogObject` included in this product set.
    All objects in this set must be included in an order for a pricing rule to apply.
    
    Only one of `product_ids_all`, `product_ids_any`, or `all_products` can be set.
    
    Max: 500 catalog object IDs.
    """

    quantity_exact: typing.Optional[int] = pydantic.Field(default=None)
    """
    If set, there must be exactly this many items from `products_any` or `products_all`
    in the cart for the discount to apply.
    
    Cannot be combined with either `quantity_min` or `quantity_max`.
    """

    quantity_min: typing.Optional[int] = pydantic.Field(default=None)
    """
    If set, there must be at least this many items from `products_any` or `products_all`
    in a cart for the discount to apply. See `quantity_exact`. Defaults to 0 if
    `quantity_exact`, `quantity_min` and `quantity_max` are all unspecified.
    """

    quantity_max: typing.Optional[int] = pydantic.Field(default=None)
    """
    If set, the pricing rule will apply to a maximum of this many items from
    `products_any` or `products_all`.
    """

    all_products: typing.Optional[bool] = pydantic.Field(default=None)
    """
    If set to `true`, the product set will include every item in the catalog.
    Only one of `product_ids_all`, `product_ids_any`, or `all_products` can be set.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from .device_attributes import DeviceAttributes
from .component import Component
from .device_status import DeviceStatus
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class Device(UncheckedBaseModel):
    id: typing.Optional[str] = pydantic.Field(default=None)
    """
    A synthetic identifier for the device. The identifier includes a standardized prefix and
    is otherwise an opaque id generated from key device fields.
    """

    attributes: DeviceAttributes = pydantic.Field()
    """
    A collection of DeviceAttributes representing the device.
    """

    components: typing.Optional[typing.List[Component]] = pydantic.Field(default=None)
    """
    A list of components applicable to the device.
    """

    status: typing.Optional[DeviceStatus] = pydantic.Field(default=None)
    """
    The current status of the device.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions


class LoyaltyAccountExpiringPointDeadlineParams(typing_extensions.TypedDict):
    """
    Represents a set of points for a loyalty account that are scheduled to expire on a specific date.
    """

    points: int
    """
    The number of points scheduled to expire at the `expires_at` timestamp.
    """

    expires_at: str
    """
    The timestamp of when the points are scheduled to expire, in RFC 3339 format.
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import pydantic
import typing
from .tip_settings import TipSettings
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class DeviceCheckoutOptions(UncheckedBaseModel):
    device_id: str = pydantic.Field()
    """
    The unique ID of the device intended for this `TerminalCheckout`.
    A list of `DeviceCode` objects can be retrieved from the /v2/devices/codes endpoint.
    Match a `DeviceCode.device_id` value with `device_id` to get the associated device code.
    """

    skip_receipt_screen: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Instructs the device to skip the receipt screen. Defaults to false.
    """

    collect_signature: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Indicates that signature collection is desired during checkout. Defaults to false.
    """

    tip_settings: typing.Optional[TipSettings] = pydantic.Field(default=None)
    """
    Tip-specific settings.
    """

    show_itemized_cart: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Show the itemization screen prior to taking a payment. This field is only meaningful when the
    checkout includes an order ID. Defaults to true.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
from ..core.client_wrapper import SyncClientWrapper
from .raw_client import RawPaymentsClient
from ..types.list_payments_request_sort_field import ListPaymentsRequestSortField
from ..core.request_options import RequestOptions
from ..core.pagination import SyncPager
from ..types.payment import Payment
from ..types.list_payments_response import ListPaymentsResponse
from ..core.unchecked_base_model import construct_type
from json.decoder import JSONDecodeError
from ..core.api_error import ApiError
from ..requests.money import MoneyParams
from ..requests.address import AddressParams
from ..requests.cash_payment_details import CashPaymentDetailsParams
from ..requests.external_payment_details import ExternalPaymentDetailsParams
from ..requests.customer_details import CustomerDetailsParams
from ..requests.offline_payment_details import OfflinePaymentDetailsParams
from ..types.create_payment_response import CreatePaymentResponse
from ..types.cancel_payment_by_idempotency_key_response import CancelPaymentByIdempotencyKeyResponse
from ..types.get_payment_response import GetPaymentResponse
from ..requests.payment import PaymentParams
from ..types.update_payment_response import UpdatePaymentResponse
from ..types.cancel_payment_response import CancelPaymentResponse
from ..types.complete_payment_response import CompletePaymentResponse
from ..core.client_wrapper import AsyncClientWrapper
from .raw_client import AsyncRawPaymentsClient
from ..core.pagination import AsyncPager

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class PaymentsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawPaymentsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawPaymentsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawPaymentsClient
        """
        return self._raw_client

    def list(
        self,
        *,
        begin_time: typing.Optional[str] = None,
        end_time: typing.Optional[str] = None,
        sort_order: typing.Optional[str] = None,
        cursor: typing.Optional[str] = None,
        location_id: typing.Optional[str] = None,
        total: typing.Optional[int] = None,
        last4: typing.Optional[str] = None,
        card_brand: typing.Optional[str] = None,
        limit: typing.Optional[int] = None,
        is_offline_payment: typing.Optional[bool] = None,
        offline_begin_time: typing.Optional[str] = None,
        offline_end_time: typing.Optional[str] = None,
        updated_at_begin_time: typing.Optional[str] = None,
        updated_at_end_time: typing.Optional[str] = None,
        sort_field: typing.Optional[ListPaymentsRequestSortField] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[Payment]:
        """
        Retrieves a list of payments taken by the account making the request.

        Results are eventually consistent, and new payments or changes to payments might take several
        seconds to appear.

        The maximum results per page is 100.

        Parameters
        ----------
        begin_time : typing.Optional[str]
            Indicates the start of the time range to retrieve payments for, in RFC 3339 format.
            The range is determined using the `created_at` field for each Payment.
            Inclusive. Default: The current time minus one year.

        end_time : typing.Optional[str]
            Indicates the end of the time range to retrieve payments for, in RFC 3339 format.  The
            range is determined using the `created_at` field for each Payment.

            Default: The current time.

        sort_order : typing.Optional[str]
            The order in which results are listed by `ListPaymentsRequest.sort_field`:
            - `ASC` - Oldest to newest.
            - `DESC` - Newest to oldest (default).

        cursor : typing.Optional[str]
            A pagination cursor returned by a previous call to this endpoint.
            Provide this cursor to retrieve the next set of results for the original query.

            For more information, see [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination).

        location_id : typing.Optional[str]
            Limit results to the location supplied. By default, results are returned
            for the default (main) location associated with the seller.

        total : typing.Optional[int]
            The exact amount in the `total_money` for a payment.

        last4 : typing.Optional[str]
            The last four digits of a payment card.

        card_brand : typing.Optional[str]
            The brand of the payment card (for example, VISA).

        limit : typing.Optional[int]
            The maximum number of results to be returned in a single page.
            It is possible to receive fewer results than the specified limit on a given page.

            The default value of 100 is also the maximum allowed value. If the provided value is
            greater than 100, it is ignored and the default value is used instead.

            Default: `100`

        is_offline_payment : typing.Optional[bool]
            Whether the payment was taken offline or not.

        offline_begin_time : typing.Optional[str]
            Indicates the start of the time range for which to retrieve offline payments, in RFC 3339
            format for timestamps. The range is determined using the
            `offline_payment_details.client_created_at` field for each Payment. If set, payments without a
            value set in `offline_payment_details.client_created_at` will not be returned.

            Default: The current time.

        offline_end_time : typing.Optional[str]
            Indicates the end of the time range for which to retrieve offline payments, in RFC 3339
            format for timestamps. The range is determined using the
            `offline_payment_details.client_created_at` field for each Payment. If set, payments without a
            value set in `offline_payment_details.client_created_at` will not be returned.

            Default: The current time.

        updated_at_begin_time : typing.Optional[str]
            Indicates the start of the time range to retrieve payments for, in RFC 3339 format.  The
            range is determined using the `updated_at` field for each Payment.

        updated_at_end_time : typing.Optional[str]
            Indicates the end of the time range to retrieve payments for, in RFC 3339 format.  The
            range is determined using the `updated_at` field for each Payment.

        sort_field : typing.Optional[ListPaymentsRequestSortField]
            The field used to sort results by. The default is `CREATED_AT`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[Payment]
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        response = client.payments.list()
        for item in response:
            yield item
        # alternatively, you can paginate page-by-page
        for page in response.iter_pages():
            yield page
        """
        _response = self._raw_client._client_wrapper.httpx_client.request(
            "v2/payments",
            method="GET",
            params={
                "begin_time": begin_time,
                "end_time": end_time,
                "sort_order": sort_order,
                "cursor": cursor,
                "location_id": location_id,
                "total": total,
                "last_4": last4,
                "card_brand": card_brand,
                "limit": limit,
                "is_offline_payment": is_offline_payment,
                "offline_begin_time": offline_begin_time,
                "offline_end_time": offline_end_time,
                "updated_at_begin_time": updated_at_begin_time,
                "updated_at_end_time": updated_at_end_time,
                "sort_field": sort_field,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    ListPaymentsResponse,
                    construct_type(
                        type_=ListPaymentsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _parsed_next = _parsed_response.cursor
                _has_next = _parsed_next is not None and _parsed_next != ""
                _get_next = lambda: self.list(
                    begin_time=begin_time,
                    end_time=end_time,
                    sort_order=sort_order,
                    cursor=_parsed_next,
                    location_id=location_id,
                    total=total,
                    last4=last4,
                    card_brand=card_brand,
                    limit=limit,
                    is_offline_payment=is_offline_payment,
                    offline_begin_time=offline_begin_time,
                    offline_end_time=offline_end_time,
                    updated_at_begin_time=updated_at_begin_time,
                    updated_at_end_time=updated_at_end_time,
                    sort_field=sort_field,
                    request_options=request_options,
                )
                _items = _parsed_response.payments
                return SyncPager(has_next=_has_next, items=_items, get_next=_get_next)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def create(
        self,
        *,
        source_id: str,
        idempotency_key: str,
        amount_money: typing.Optional[MoneyParams] = OMIT,
        tip_money: typing.Optional[MoneyParams] = OMIT,
        app_fee_money: typing.Optional[MoneyParams] = OMIT,
        delay_duration: typing.Optional[str] = OMIT,
        delay_action: typing.Optional[str] = OMIT,
        autocomplete: typing.Optional[bool] = OMIT,
        order_id: typing.Optional[str] = OMIT,
        customer_id: typing.Optional[str] = OMIT,
        location_id: typing.Optional[str] = OMIT,
        team_member_id: typing.Optional[str] = OMIT,
        reference_id: typing.Optional[str] = OMIT,
        verification_token: typing.Optional[str] = OMIT,
        accept_partial_authorization: typing.Optional[bool] = OMIT,
        buyer_email_address: typing.Optional[str] = OMIT,
        buyer_phone_number: typing.Optional[str] = OMIT,
        billing_address: typing.Optional[AddressParams] = OMIT,
        shipping_address: typing.Optional[AddressParams] = OMIT,
        note: typing.Optional[str] = OMIT,
        statement_description_identifier: typing.Optional[str] = OMIT,
        cash_details: typing.Optional[CashPaymentDetailsParams] = OMIT,
        external_details: typing.Optional[ExternalPaymentDetailsParams] = OMIT,
        customer_details: typing.Optional[CustomerDetailsParams] = OMIT,
        offline_payment_details: typing.Optional[OfflinePaymentDetailsParams] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreatePaymentResponse:
        """
        Creates a payment using the provided source. You can use this endpoint
        to charge a card (credit/debit card or
        Square gift card) or record a payment that the seller received outside of Square
        (cash payment from a buyer or a payment that an external entity
        processed on behalf of the seller).

        The endpoint creates a
        `Payment` object and returns it in the response.

        Parameters
        ----------
        source_id : str
            The ID for the source of funds for this payment.
            This could be a payment token generated by the Web Payments SDK for any of its
            [supported methods](https://developer.squareup.com/docs/web-payments/overview#explore-payment-methods),
            including cards, bank transfers, Afterpay or Cash App Pay. If recording a payment
            that the seller received outside of Square, specify either "CASH" or "EXTERNAL".
            For more information, see
            [Take Payments](https://developer.squareup.com/docs/payments-api/take-payments).

        idempotency_key : str
            A unique string that identifies this `CreatePayment` request. Keys can be any valid string
            but must be unique for every `CreatePayment` request.

            Note: The number of allowed characters might be less than the stated maximum, if multi-byte
            characters are used.

            For more information, see [Idempotency](https://developer.squareup.com/docs/working-with-apis/idempotency).

        amount_money : typing.Optional[MoneyParams]
            The amount of money to accept for this payment, not including `tip_money`.

            The amount must be specified in the smallest denomination of the applicable currency
            (for example, US dollar amounts are specified in cents). For more information, see
            [Working with Monetary Amounts](https://developer.squareup.com/docs/build-basics/working-with-monetary-amounts).

            The currency code must match the currency associated with the business
            that is accepting the payment.

        tip_money : typing.Optional[MoneyParams]
            The amount designated as a tip, in addition to `amount_money`.

            The amount must be specified in the smallest denomination of the applicable currency
            (for example, US dollar amounts are specified in cents). For more information, see
            [Working with Monetary Amounts](https://developer.squareup.com/docs/build-basics/working-with-monetary-amounts).

            The currency code must match the currency associated with the business
            that is accepting the payment.

        app_fee_money : typing.Optional[MoneyParams]
            The amount of money that the developer is taking as a fee
            for facilitating the payment on behalf of the seller.

            The amount cannot be more than 90% of the total amount of the payment.

            The amount must be specified in the smallest denomination of the applicable currency
            (for example, US dollar amounts are specified in cents). For more information, see
            [Working with Monetary Amounts](https://developer.squareup.com/docs/build-basics/working-with-monetary-amounts).

            The fee currency code must match the currency associated with the seller
            that is accepting the payment. The application must be from a developer
            account in the same country and using the same currency code as the seller.

            For more information about the application fee scenario, see
            [Take Payments and Collect Fees](https://developer.squareup.com/docs/payments-api/take-payments-and-collect-fees).

            To set this field, `PAYMENTS_WRITE_ADDITIONAL_RECIPIENTS` OAuth permission is required.
            For more information, see [Permissions](https://developer.squareup.com/docs/payments-api/take-payments-and-collect-fees#permissions).

        delay_duration : typing.Optional[str]
            The duration of time after the payment's creation when Square automatically
            either completes or cancels the payment depending on the `delay_action` field value.
            For more information, see
            [Time threshold](https://developer.squareup.com/docs/payments-api/take-payments/card-payments/delayed-capture#time-threshold).

            This parameter should be specified as a time duration, in RFC 3339 format.

            Note: This feature is only supported for card payments. This parameter can only be set for a delayed
            capture payment (`autocomplete=false`).

            Default:

            - Card-present payments: "PT36H" (36 hours) from the creation time.
            - Card-not-present payments: "P7D" (7 days) from the creation time.

        delay_action : typing.Optional[str]
            The action to be applied to the payment when the `delay_duration` has elapsed. The action must be
            CANCEL or COMPLETE. For more information, see
            [Time Threshold](https://developer.squareup.com/docs/payments-api/take-payments/card-payments/delayed-capture#time-threshold).

            Default: CANCEL

        autocomplete : typing.Optional[bool]
            If set to `true`, this payment will be completed when possible. If
            set to `false`, this payment is held in an approved state until either
            explicitly completed (captured) or canceled (voided). For more information, see
            [Delayed capture](https://developer.squareup.com/docs/payments-api/take-payments/card-payments#delayed-capture-of-a-card-payment).

            Default: true

        order_id : typing.Optional[str]
            Associates a previously created order with this payment.

        customer_id : typing.Optional[str]
            The [Customer](entity:Customer) ID of the customer associated with the payment.

            This is required if the `source_id` refers to a card on file created using the Cards API.

        location_id : typing.Optional[str]
            The location ID to associate with the payment. If not specified, the [main location](https://developer.squareup.com/docs/locations-api#about-the-main-location) is
            used.

        team_member_id : typing.Optional[str]
            An optional [TeamMember](entity:TeamMember) ID to associate with
            this payment.

        reference_id : typing.Optional[str]
            A user-defined ID to associate with the payment.

            You can use this field to associate the payment to an entity in an external system
            (for example, you might specify an order ID that is generated by a third-party shopping cart).

        verification_token : typing.Optional[str]
            An identifying token generated by [payments.verifyBuyer()](https://developer.squareup.com/reference/sdks/web/payments/objects/Payments#Payments.verifyBuyer).
            Verification tokens encapsulate customer device information and 3-D Secure
            challenge results to indicate that Square has verified the buyer identity.

            For more information, see [SCA Overview](https://developer.squareup.com/docs/sca-overview).

        accept_partial_authorization : typing.Optional[bool]
            If set to `true` and charging a Square Gift Card, a payment might be returned with
            `amount_money` equal to less than what was requested. For example, a request for $20 when charging
            a Square Gift Card with a balance of $5 results in an APPROVED payment of $5. You might choose
            to prompt the buyer for an additional payment to cover the remainder or cancel the Gift Card
            payment. This field cannot be `true` when `autocomplete = true`.

            For more information, see
            [Partial amount with Square Gift Cards](https://developer.squareup.com/docs/payments-api/take-payments#partial-payment-gift-card).

            Default: false

        buyer_email_address : typing.Optional[str]
            The buyer's email address.

        buyer_phone_number : typing.Optional[str]
            The buyer's phone number.
            Must follow the following format:
            1. A leading + symbol (followed by a country code)
            2. The phone number can contain spaces and the special characters `(` , `)` , `-` , and `.`.
            Alphabetical characters aren't allowed.
            3. The phone number must contain between 9 and 16 digits.

        billing_address : typing.Optional[AddressParams]
            The buyer's billing address.

        shipping_address : typing.Optional[AddressParams]
            The buyer's shipping address.

        note : typing.Optional[str]
            An optional note to be entered by the developer when creating a payment.

        statement_description_identifier : typing.Optional[str]
            Optional additional payment information to include on the customer's card statement
            as part of the statement description. This can be, for example, an invoice number, ticket number,
            or short description that uniquely identifies the purchase.

            Note that the `statement_description_identifier` might get truncated on the statement description
            to fit the required information including the Square identifier (SQ *) and name of the
            seller taking the payment.

        cash_details : typing.Optional[CashPaymentDetailsParams]
            Additional details required when recording a cash payment (`source_id` is CASH).

        external_details : typing.Optional[ExternalPaymentDetailsParams]
            Additional details required when recording an external payment (`source_id` is EXTERNAL).

        customer_details : typing.Optional[CustomerDetailsParams]
            Details about the customer making the payment.

        offline_payment_details : typing.Optional[OfflinePaymentDetailsParams]
            An optional field for specifying the offline payment details. This is intended for
            internal 1st-party callers only.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreatePaymentResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.payments.create(
            source_id="ccof:GaJGNaZa8x4OgDJn4GB",
            idempotency_key="7b0f3ec5-086a-4871-8f13-3c81b3875218",
            amount_money={"amount": 1000, "currency": "USD"},
            app_fee_money={"amount": 10, "currency": "USD"},
            autocomplete=True,
            customer_id="W92WH6P11H4Z77CTET0RNTGFW8",
            location_id="L88917AVBK2S5",
            reference_id="123456",
            note="Brief description",
        )
        """
        response = self._raw_client.create(
            source_id=source_id,
            idempotency_key=idempotency_key,
            amount_money=amount_money,
            tip_money=tip_money,
            app_fee_money=app_fee_money,
            delay_duration=delay_duration,
            delay_action=delay_action,
            autocomplete=autocomplete,
            order_id=order_id,
            customer_id=customer_id,
            location_id=location_id,
            team_member_id=team_member_id,
            reference_id=reference_id,
            verification_token=verification_token,
            accept_partial_authorization=accept_partial_authorization,
            buyer_email_address=buyer_email_address,
            buyer_phone_number=buyer_phone_number,
            billing_address=billing_address,
            shipping_address=shipping_address,
            note=note,
            statement_description_identifier=statement_description_identifier,
            cash_details=cash_details,
            external_details=external_details,
            customer_details=customer_details,
            offline_payment_details=offline_payment_details,
            request_options=request_options,
        )
        return response.data

    def cancel_by_idempotency_key(
        self, *, idempotency_key: str, request_options: typing.Optional[RequestOptions] = None
    ) -> CancelPaymentByIdempotencyKeyResponse:
        """
        Cancels (voids) a payment identified by the idempotency key that is specified in the
        request.

        Use this method when the status of a `CreatePayment` request is unknown (for example, after you send a
        `CreatePayment` request, a network error occurs and you do not get a response). In this case, you can
        direct Square to cancel the payment using this endpoint. In the request, you provide the same
        idempotency key that you provided in your `CreatePayment` request that you want to cancel. After
        canceling the payment, you can submit your `CreatePayment` request again.

        Note that if no payment with the specified idempotency key is found, no action is taken and the endpoint
        returns successfully.

        Parameters
        ----------
        idempotency_key : str
            The `idempotency_key` identifying the payment to be canceled.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CancelPaymentByIdempotencyKeyResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.payments.cancel_by_idempotency_key(
            idempotency_key="a7e36d40-d24b-11e8-b568-0800200c9a66",
        )
        """
        response = self._raw_client.cancel_by_idempotency_key(
            idempotency_key=idempotency_key, request_options=request_options
        )
        return response.data

    def get(self, payment_id: str, *, request_options: typing.Optional[RequestOptions] = None) -> GetPaymentResponse:
        """
        Retrieves details for a specific payment.

        Parameters
        ----------
        payment_id : str
            A unique ID for the desired payment.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetPaymentResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.payments.get(
            payment_id="payment_id",
        )
        """
        response = self._raw_client.get(payment_id, request_options=request_options)
        return response.data

    def update(
        self,
        payment_id: str,
        *,
        idempotency_key: str,
        payment: typing.Optional[PaymentParams] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdatePaymentResponse:
        """
        Updates a payment with the APPROVED status.
        You can update the `amount_money` and `tip_money` using this endpoint.

        Parameters
        ----------
        payment_id : str
            The ID of the payment to update.

        idempotency_key : str
            A unique string that identifies this `UpdatePayment` request. Keys can be any valid string
            but must be unique for every `UpdatePayment` request.

            For more information, see [Idempotency](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency).

        payment : typing.Optional[PaymentParams]
            The updated `Payment` object.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdatePaymentResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.payments.update(
            payment_id="payment_id",
            payment={
                "amount_money": {"amount": 1000, "currency": "USD"},
                "tip_money": {"amount": 100, "currency": "USD"},
                "version_token": "ODhwVQ35xwlzRuoZEwKXucfu7583sPTzK48c5zoGd0g6o",
            },
            idempotency_key="956f8b13-e4ec-45d6-85e8-d1d95ef0c5de",
        )
        """
        response = self._raw_client.update(
            payment_id, idempotency_key=idempotency_key, payment=payment, request_options=request_options
        )
        return response.data

    def cancel(
        self, payment_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> CancelPaymentResponse:
        """
        Cancels (voids) a payment. You can use this endpoint to cancel a payment with
        the APPROVED `status`.

        Parameters
        ----------
        payment_id : str
            The ID of the payment to cancel.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CancelPaymentResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.payments.cancel(
            payment_id="payment_id",
        )
        """
        response = self._raw_client.cancel(payment_id, request_options=request_options)
        return response.data

    def complete(
        self,
        payment_id: str,
        *,
        version_token: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CompletePaymentResponse:
        """
        Completes (captures) a payment.
        By default, payments are set to complete immediately after they are created.

        You can use this endpoint to complete a payment with the APPROVED `status`.

        Parameters
        ----------
        payment_id : str
            The unique ID identifying the payment to be completed.

        version_token : typing.Optional[str]
            Used for optimistic concurrency. This opaque token identifies the current `Payment`
            version that the caller expects. If the server has a different version of the Payment,
            the update fails and a response with a VERSION_MISMATCH error is returned.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CompletePaymentResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.payments.complete(
            payment_id="payment_id",
        )
        """
        response = self._raw_client.complete(payment_id, version_token=version_token, request_options=request_options)
        return response.data


class AsyncPaymentsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawPaymentsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawPaymentsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawPaymentsClient
        """
        return self._raw_client

    async def list(
        self,
        *,
        begin_time: typing.Optional[str] = None,
        end_time: typing.Optional[str] = None,
        sort_order: typing.Optional[str] = None,
        cursor: typing.Optional[str] = None,
        location_id: typing.Optional[str] = None,
        total: typing.Optional[int] = None,
        last4: typing.Optional[str] = None,
        card_brand: typing.Optional[str] = None,
        limit: typing.Optional[int] = None,
        is_offline_payment: typing.Optional[bool] = None,
        offline_begin_time: typing.Optional[str] = None,
        offline_end_time: typing.Optional[str] = None,
        updated_at_begin_time: typing.Optional[str] = None,
        updated_at_end_time: typing.Optional[str] = None,
        sort_field: typing.Optional[ListPaymentsRequestSortField] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[Payment]:
        """
        Retrieves a list of payments taken by the account making the request.

        Results are eventually consistent, and new payments or changes to payments might take several
        seconds to appear.

        The maximum results per page is 100.

        Parameters
        ----------
        begin_time : typing.Optional[str]
            Indicates the start of the time range to retrieve payments for, in RFC 3339 format.
            The range is determined using the `created_at` field for each Payment.
            Inclusive. Default: The current time minus one year.

        end_time : typing.Optional[str]
            Indicates the end of the time range to retrieve payments for, in RFC 3339 format.  The
            range is determined using the `created_at` field for each Payment.

            Default: The current time.

        sort_order : typing.Optional[str]
            The order in which results are listed by `ListPaymentsRequest.sort_field`:
            - `ASC` - Oldest to newest.
            - `DESC` - Newest to oldest (default).

        cursor : typing.Optional[str]
            A pagination cursor returned by a previous call to this endpoint.
            Provide this cursor to retrieve the next set of results for the original query.

            For more information, see [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination).

        location_id : typing.Optional[str]
            Limit results to the location supplied. By default, results are returned
            for the default (main) location associated with the seller.

        total : typing.Optional[int]
            The exact amount in the `total_money` for a payment.

        last4 : typing.Optional[str]
            The last four digits of a payment card.

        card_brand : typing.Optional[str]
            The brand of the payment card (for example, VISA).

        limit : typing.Optional[int]
            The maximum number of results to be returned in a single page.
            It is possible to receive fewer results than the specified limit on a given page.

            The default value of 100 is also the maximum allowed value. If the provided value is
            greater than 100, it is ignored and the default value is used instead.

            Default: `100`

        is_offline_payment : typing.Optional[bool]
            Whether the payment was taken offline or not.

        offline_begin_time : typing.Optional[str]
            Indicates the start of the time range for which to retrieve offline payments, in RFC 3339
            format for timestamps. The range is determined using the
            `offline_payment_details.client_created_at` field for each Payment. If set, payments without a
            value set in `offline_payment_details.client_created_at` will not be returned.

            Default: The current time.

        offline_end_time : typing.Optional[str]
            Indicates the end of the time range for which to retrieve offline payments, in RFC 3339
            format for timestamps. The range is determined using the
            `offline_payment_details.client_created_at` field for each Payment. If set, payments without a
            value set in `offline_payment_details.client_created_at` will not be returned.

            Default: The current time.

        updated_at_begin_time : typing.Optional[str]
            Indicates the start of the time range to retrieve payments for, in RFC 3339 format.  The
            range is determined using the `updated_at` field for each Payment.

        updated_at_end_time : typing.Optional[str]
            Indicates the end of the time range to retrieve payments for, in RFC 3339 format.  The
            range is determined using the `updated_at` field for each Payment.

        sort_field : typing.Optional[ListPaymentsRequestSortField]
            The field used to sort results by. The default is `CREATED_AT`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[Payment]
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            response = await client.payments.list()
            async for item in response:
                yield item
            # alternatively, you can paginate page-by-page
            async for page in response.iter_pages():
                yield page


        asyncio.run(main())
        """
        _response = await self._raw_client._client_wrapper.httpx_client.request(
            "v2/payments",
            method="GET",
            params={
                "begin_time": begin_time,
                "end_time": end_time,
                "sort_order": sort_order,
                "cursor": cursor,
                "location_id": location_id,
                "total": total,
                "last_4": last4,
                "card_brand": card_brand,
                "limit": limit,
                "is_offline_payment": is_offline_payment,
                "offline_begin_time": offline_begin_time,
                "offline_end_time": offline_end_time,
                "updated_at_begin_time": updated_at_begin_time,
                "updated_at_end_time": updated_at_end_time,
                "sort_field": sort_field,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    ListPaymentsResponse,
                    construct_type(
                        type_=ListPaymentsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _parsed_next = _parsed_response.cursor
                _has_next = _parsed_next is not None and _parsed_next != ""
                _get_next = lambda: self.list(
                    begin_time=begin_time,
                    end_time=end_time,
                    sort_order=sort_order,
                    cursor=_parsed_next,
                    location_id=location_id,
                    total=total,
                    last4=last4,
                    card_brand=card_brand,
                    limit=limit,
                    is_offline_payment=is_offline_payment,
                    offline_begin_time=offline_begin_time,
                    offline_end_time=offline_end_time,
                    updated_at_begin_time=updated_at_begin_time,
                    updated_at_end_time=updated_at_end_time,
                    sort_field=sort_field,
                    request_options=request_options,
                )
                _items = _parsed_response.payments
                return AsyncPager(has_next=_has_next, items=_items, get_next=_get_next)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def create(
        self,
        *,
        source_id: str,
        idempotency_key: str,
        amount_money: typing.Optional[MoneyParams] = OMIT,
        tip_money: typing.Optional[MoneyParams] = OMIT,
        app_fee_money: typing.Optional[MoneyParams] = OMIT,
        delay_duration: typing.Optional[str] = OMIT,
        delay_action: typing.Optional[str] = OMIT,
        autocomplete: typing.Optional[bool] = OMIT,
        order_id: typing.Optional[str] = OMIT,
        customer_id: typing.Optional[str] = OMIT,
        location_id: typing.Optional[str] = OMIT,
        team_member_id: typing.Optional[str] = OMIT,
        reference_id: typing.Optional[str] = OMIT,
        verification_token: typing.Optional[str] = OMIT,
        accept_partial_authorization: typing.Optional[bool] = OMIT,
        buyer_email_address: typing.Optional[str] = OMIT,
        buyer_phone_number: typing.Optional[str] = OMIT,
        billing_address: typing.Optional[AddressParams] = OMIT,
        shipping_address: typing.Optional[AddressParams] = OMIT,
        note: typing.Optional[str] = OMIT,
        statement_description_identifier: typing.Optional[str] = OMIT,
        cash_details: typing.Optional[CashPaymentDetailsParams] = OMIT,
        external_details: typing.Optional[ExternalPaymentDetailsParams] = OMIT,
        customer_details: typing.Optional[CustomerDetailsParams] = OMIT,
        offline_payment_details: typing.Optional[OfflinePaymentDetailsParams] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreatePaymentResponse:
        """
        Creates a payment using the provided source. You can use this endpoint
        to charge a card (credit/debit card or
        Square gift card) or record a payment that the seller received outside of Square
        (cash payment from a buyer or a payment that an external entity
        processed on behalf of the seller).

        The endpoint creates a
        `Payment` object and returns it in the response.

        Parameters
        ----------
        source_id : str
            The ID for the source of funds for this payment.
            This could be a payment token generated by the Web Payments SDK for any of its
            [supported methods](https://developer.squareup.com/docs/web-payments/overview#explore-payment-methods),
            including cards, bank transfers, Afterpay or Cash App Pay. If recording a payment
            that the seller received outside of Square, specify either "CASH" or "EXTERNAL".
            For more information, see
            [Take Payments](https://developer.squareup.com/docs/payments-api/take-payments).

        idempotency_key : str
            A unique string that identifies this `CreatePayment` request. Keys can be any valid string
            but must be unique for every `CreatePayment` request.

            Note: The number of allowed characters might be less than the stated maximum, if multi-byte
            characters are used.

            For more information, see [Idempotency](https://developer.squareup.com/docs/working-with-apis/idempotency).

        amount_money : typing.Optional[MoneyParams]
            The amount of money to accept for this payment, not including `tip_money`.

            The amount must be specified in the smallest denomination of the applicable currency
            (for example, US dollar amounts are specified in cents). For more information, see
            [Working with Monetary Amounts](https://developer.squareup.com/docs/build-basics/working-with-monetary-amounts).

            The currency code must match the currency associated with the business
            that is accepting the payment.

        tip_money : typing.Optional[MoneyParams]
            The amount designated as a tip, in addition to `amount_money`.

            The amount must be specified in the smallest denomination of the applicable currency
            (for example, US dollar amounts are specified in cents). For more information, see
            [Working with Monetary Amounts](https://developer.squareup.com/docs/build-basics/working-with-monetary-amounts).

            The currency code must match the currency associated with the business
            that is accepting the payment.

        app_fee_money : typing.Optional[MoneyParams]
            The amount of money that the developer is taking as a fee
            for facilitating the payment on behalf of the seller.

            The amount cannot be more than 90% of the total amount of the payment.

            The amount must be specified in the smallest denomination of the applicable currency
            (for example, US dollar amounts are specified in cents). For more information, see
            [Working with Monetary Amounts](https://developer.squareup.com/docs/build-basics/working-with-monetary-amounts).

            The fee currency code must match the currency associated with the seller
            that is accepting the payment. The application must be from a developer
            account in the same country and using the same currency code as the seller.

            For more information about the application fee scenario, see
            [Take Payments and Collect Fees](https://developer.squareup.com/docs/payments-api/take-payments-and-collect-fees).

            To set this field, `PAYMENTS_WRITE_ADDITIONAL_RECIPIENTS` OAuth permission is required.
            For more information, see [Permissions](https://developer.squareup.com/docs/payments-api/take-payments-and-collect-fees#permissions).

        delay_duration : typing.Optional[str]
            The duration of time after the payment's creation when Square automatically
            either completes or cancels the payment depending on the `delay_action` field value.
            For more information, see
            [Time threshold](https://developer.squareup.com/docs/payments-api/take-payments/card-payments/delayed-capture#time-threshold).

            This parameter should be specified as a time duration, in RFC 3339 format.

            Note: This feature is only supported for card payments. This parameter can only be set for a delayed
            capture payment (`autocomplete=false`).

            Default:

            - Card-present payments: "PT36H" (36 hours) from the creation time.
            - Card-not-present payments: "P7D" (7 days) from the creation time.

        delay_action : typing.Optional[str]
            The action to be applied to the payment when the `delay_duration` has elapsed. The action must be
            CANCEL or COMPLETE. For more information, see
            [Time Threshold](https://developer.squareup.com/docs/payments-api/take-payments/card-payments/delayed-capture#time-threshold).

            Default: CANCEL

        autocomplete : typing.Optional[bool]
            If set to `true`, this payment will be completed when possible. If
            set to `false`, this payment is held in an approved state until either
            explicitly completed (captured) or canceled (voided). For more information, see
            [Delayed capture](https://developer.squareup.com/docs/payments-api/take-payments/card-payments#delayed-capture-of-a-card-payment).

            Default: true

        order_id : typing.Optional[str]
            Associates a previously created order with this payment.

        customer_id : typing.Optional[str]
            The [Customer](entity:Customer) ID of the customer associated with the payment.

            This is required if the `source_id` refers to a card on file created using the Cards API.

        location_id : typing.Optional[str]
            The location ID to associate with the payment. If not specified, the [main location](https://developer.squareup.com/docs/locations-api#about-the-main-location) is
            used.

        team_member_id : typing.Optional[str]
            An optional [TeamMember](entity:TeamMember) ID to associate with
            this payment.

        reference_id : typing.Optional[str]
            A user-defined ID to associate with the payment.

            You can use this field to associate the payment to an entity in an external system
            (for example, you might specify an order ID that is generated by a third-party shopping cart).

        verification_token : typing.Optional[str]
            An identifying token generated by [payments.verifyBuyer()](https://developer.squareup.com/reference/sdks/web/payments/objects/Payments#Payments.verifyBuyer).
            Verification tokens encapsulate customer device information and 3-D Secure
            challenge results to indicate that Square has verified the buyer identity.

            For more information, see [SCA Overview](https://developer.squareup.com/docs/sca-overview).

        accept_partial_authorization : typing.Optional[bool]
            If set to `true` and charging a Square Gift Card, a payment might be returned with
            `amount_money` equal to less than what was requested. For example, a request for $20 when charging
            a Square Gift Card with a balance of $5 results in an APPROVED payment of $5. You might choose
            to prompt the buyer for an additional payment to cover the remainder or cancel the Gift Card
            payment. This field cannot be `true` when `autocomplete = true`.

            For more information, see
            [Partial amount with Square Gift Cards](https://developer.squareup.com/docs/payments-api/take-payments#partial-payment-gift-card).

            Default: false

        buyer_email_address : typing.Optional[str]
            The buyer's email address.

        buyer_phone_number : typing.Optional[str]
            The buyer's phone number.
            Must follow the following format:
            1. A leading + symbol (followed by a country code)
            2. The phone number can contain spaces and the special characters `(` , `)` , `-` , and `.`.
            Alphabetical characters aren't allowed.
            3. The phone number must contain between 9 and 16 digits.

        billing_address : typing.Optional[AddressParams]
            The buyer's billing address.

        shipping_address : typing.Optional[AddressParams]
            The buyer's shipping address.

        note : typing.Optional[str]
            An optional note to be entered by the developer when creating a payment.

        statement_description_identifier : typing.Optional[str]
            Optional additional payment information to include on the customer's card statement
            as part of the statement description. This can be, for example, an invoice number, ticket number,
            or short description that uniquely identifies the purchase.

            Note that the `statement_description_identifier` might get truncated on the statement description
            to fit the required information including the Square identifier (SQ *) and name of the
            seller taking the payment.

        cash_details : typing.Optional[CashPaymentDetailsParams]
            Additional details required when recording a cash payment (`source_id` is CASH).

        external_details : typing.Optional[ExternalPaymentDetailsParams]
            Additional details required when recording an external payment (`source_id` is EXTERNAL).

        customer_details : typing.Optional[CustomerDetailsParams]
            Details about the customer making the payment.

        offline_payment_details : typing.Optional[OfflinePaymentDetailsParams]
            An optional field for specifying the offline payment details. This is intended for
            internal 1st-party callers only.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreatePaymentResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.payments.create(
                source_id="ccof:GaJGNaZa8x4OgDJn4GB",
                idempotency_key="7b0f3ec5-086a-4871-8f13-3c81b3875218",
                amount_money={"amount": 1000, "currency": "USD"},
                app_fee_money={"amount": 10, "currency": "USD"},
                autocomplete=True,
                customer_id="W92WH6P11H4Z77CTET0RNTGFW8",
                location_id="L88917AVBK2S5",
                reference_id="123456",
                note="Brief description",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.create(
            source_id=source_id,
            idempotency_key=idempotency_key,
            amount_money=amount_money,
            tip_money=tip_money,
            app_fee_money=app_fee_money,
            delay_duration=delay_duration,
            delay_action=delay_action,
            autocomplete=autocomplete,
            order_id=order_id,
            customer_id=customer_id,
            location_id=location_id,
            team_member_id=team_member_id,
            reference_id=reference_id,
            verification_token=verification_token,
            accept_partial_authorization=accept_partial_authorization,
            buyer_email_address=buyer_email_address,
            buyer_phone_number=buyer_phone_number,
            billing_address=billing_address,
            shipping_address=shipping_address,
            note=note,
            statement_description_identifier=statement_description_identifier,
            cash_details=cash_details,
            external_details=external_details,
            customer_details=customer_details,
            offline_payment_details=offline_payment_details,
            request_options=request_options,
        )
        return response.data

    async def cancel_by_idempotency_key(
        self, *, idempotency_key: str, request_options: typing.Optional[RequestOptions] = None
    ) -> CancelPaymentByIdempotencyKeyResponse:
        """
        Cancels (voids) a payment identified by the idempotency key that is specified in the
        request.

        Use this method when the status of a `CreatePayment` request is unknown (for example, after you send a
        `CreatePayment` request, a network error occurs and you do not get a response). In this case, you can
        direct Square to cancel the payment using this endpoint. In the request, you provide the same
        idempotency key that you provided in your `CreatePayment` request that you want to cancel. After
        canceling the payment, you can submit your `CreatePayment` request again.

        Note that if no payment with the specified idempotency key is found, no action is taken and the endpoint
        returns successfully.

        Parameters
        ----------
        idempotency_key : str
            The `idempotency_key` identifying the payment to be canceled.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CancelPaymentByIdempotencyKeyResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.payments.cancel_by_idempotency_key(
                idempotency_key="a7e36d40-d24b-11e8-b568-0800200c9a66",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.cancel_by_idempotency_key(
            idempotency_key=idempotency_key, request_options=request_options
        )
        return response.data

    async def get(
        self, payment_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetPaymentResponse:
        """
        Retrieves details for a specific payment.

        Parameters
        ----------
        payment_id : str
            A unique ID for the desired payment.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetPaymentResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.payments.get(
                payment_id="payment_id",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.get(payment_id, request_options=request_options)
        return response.data

    async def update(
        self,
        payment_id: str,
        *,
        idempotency_key: str,
        payment: typing.Optional[PaymentParams] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdatePaymentResponse:
        """
        Updates a payment with the APPROVED status.
        You can update the `amount_money` and `tip_money` using this endpoint.

        Parameters
        ----------
        payment_id : str
            The ID of the payment to update.

        idempotency_key : str
            A unique string that identifies this `UpdatePayment` request. Keys can be any valid string
            but must be unique for every `UpdatePayment` request.

            For more information, see [Idempotency](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency).

        payment : typing.Optional[PaymentParams]
            The updated `Payment` object.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdatePaymentResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.payments.update(
                payment_id="payment_id",
                payment={
                    "amount_money": {"amount": 1000, "currency": "USD"},
                    "tip_money": {"amount": 100, "currency": "USD"},
                    "version_token": "ODhwVQ35xwlzRuoZEwKXucfu7583sPTzK48c5zoGd0g6o",
                },
                idempotency_key="956f8b13-e4ec-45d6-85e8-d1d95ef0c5de",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.update(
            payment_id, idempotency_key=idempotency_key, payment=payment, request_options=request_options
        )
        return response.data

    async def cancel(
        self, payment_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> CancelPaymentResponse:
        """
        Cancels (voids) a payment. You can use this endpoint to cancel a payment with
        the APPROVED `status`.

        Parameters
        ----------
        payment_id : str
            The ID of the payment to cancel.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CancelPaymentResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.payments.cancel(
                payment_id="payment_id",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.cancel(payment_id, request_options=request_options)
        return response.data

    async def complete(
        self,
        payment_id: str,
        *,
        version_token: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CompletePaymentResponse:
        """
        Completes (captures) a payment.
        By default, payments are set to complete immediately after they are created.

        You can use this endpoint to complete a payment with the APPROVED `status`.

        Parameters
        ----------
        payment_id : str
            The unique ID identifying the payment to be completed.

        version_token : typing.Optional[str]
            Used for optimistic concurrency. This opaque token identifies the current `Payment`
            version that the caller expects. If the server has a different version of the Payment,
            the update fails and a response with a VERSION_MISMATCH error is returned.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CompletePaymentResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.payments.complete(
                payment_id="payment_id",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.complete(
            payment_id, version_token=version_token, request_options=request_options
        )
        return response.data



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
from .measurement_unit import MeasurementUnit
import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class StandardUnitDescription(UncheckedBaseModel):
    """
    Contains the name and abbreviation for standard measurement unit.
    """

    unit: typing.Optional[MeasurementUnit] = pydantic.Field(default=None)
    """
    Identifies the measurement unit being described.
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    UI display name of the measurement unit. For example, 'Pound'.
    """

    abbreviation: typing.Optional[str] = pydantic.Field(default=None)
    """
    UI display abbreviation for the measurement unit. For example, 'lb'.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .money import MoneyParams


class OrderLineItemAppliedDiscountParams(typing_extensions.TypedDict):
    """
    Represents an applied portion of a discount to a line item in an order.

    Order scoped discounts have automatically applied discounts present for each line item.
    Line-item scoped discounts must have applied discounts added manually for any applicable line
    items. The corresponding applied money is automatically computed based on participating
    line items.
    """

    uid: typing_extensions.NotRequired[typing.Optional[str]]
    """
    A unique ID that identifies the applied discount only within this order.
    """

    discount_uid: str
    """
    The `uid` of the discount that the applied discount represents. It must
    reference a discount present in the `order.discounts` field.
    
    This field is immutable. To change which discounts apply to a line item,
    you must delete the discount and re-add it as a new `OrderLineItemAppliedDiscount`.
    """

    applied_money: typing_extensions.NotRequired[MoneyParams]
    """
    The amount of money applied by the discount to the line item.
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class LoyaltyEventAccumulatePoints(UncheckedBaseModel):
    """
    Provides metadata when the event `type` is `ACCUMULATE_POINTS`.
    """

    loyalty_program_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The ID of the [loyalty program](entity:LoyaltyProgram).
    """

    points: typing.Optional[int] = pydantic.Field(default=None)
    """
    The number of points accumulated by the event.
    """

    order_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The ID of the [order](entity:Order) for which the buyer accumulated the points.
    This field is returned only if the Orders API is used to process orders.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class TeamMemberBookingProfile(UncheckedBaseModel):
    """
    The booking profile of a seller's team member, including the team member's ID, display name, description and whether the team member can be booked as a service provider.
    """

    team_member_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The ID of the [TeamMember](entity:TeamMember) object for the team member associated with the booking profile.
    """

    description: typing.Optional[str] = pydantic.Field(default=None)
    """
    The description of the team member.
    """

    display_name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The display name of the team member.
    """

    is_bookable: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Indicates whether the team member can be booked through the Bookings API or the seller's online booking channel or site (`true`) or not (`false`).
    """

    profile_image_url: typing.Optional[str] = pydantic.Field(default=None)
    """
    The URL of the team member's image for the bookings profile.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
from .money import MoneyParams


class OrderMoneyAmountsParams(typing_extensions.TypedDict):
    """
    A collection of various money amounts.
    """

    total_money: typing_extensions.NotRequired[MoneyParams]
    """
    The total money.
    """

    tax_money: typing_extensions.NotRequired[MoneyParams]
    """
    The money associated with taxes.
    """

    discount_money: typing_extensions.NotRequired[MoneyParams]
    """
    The money associated with discounts.
    """

    tip_money: typing_extensions.NotRequired[MoneyParams]
    """
    The money associated with tips.
    """

    service_charge_money: typing_extensions.NotRequired[MoneyParams]
    """
    The money associated with service charges.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .error import ErrorParams


class DeleteTimecardResponseParams(typing_extensions.TypedDict):
    """
    The response to a request to delete a `Timecard`. The response might contain a set of
    `Error` objects if the request resulted in errors.
    """

    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Any errors that occurred during the request.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from ..types.inventory_state import InventoryState


class BatchGetInventoryCountsRequestParams(typing_extensions.TypedDict):
    catalog_object_ids: typing_extensions.NotRequired[typing.Optional[typing.Sequence[str]]]
    """
    The filter to return results by `CatalogObject` ID.
    The filter is applicable only when set.  The default is null.
    """

    location_ids: typing_extensions.NotRequired[typing.Optional[typing.Sequence[str]]]
    """
    The filter to return results by `Location` ID.
    This filter is applicable only when set. The default is null.
    """

    updated_after: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The filter to return results with their `calculated_at` value
    after the given time as specified in an RFC 3339 timestamp.
    The default value is the UNIX epoch of (`1970-01-01T00:00:00Z`).
    """

    cursor: typing_extensions.NotRequired[typing.Optional[str]]
    """
    A pagination cursor returned by a previous call to this endpoint.
    Provide this to retrieve the next set of results for the original query.
    
    See the [Pagination](https://developer.squareup.com/docs/working-with-apis/pagination) guide for more information.
    """

    states: typing_extensions.NotRequired[typing.Optional[typing.Sequence[InventoryState]]]
    """
    The filter to return results by `InventoryState`. The filter is only applicable when set.
    Ignored are untracked states of `NONE`, `SOLD`, and `UNLINKED_RETURN`.
    The default is null.
    """

    limit: typing_extensions.NotRequired[typing.Optional[int]]
    """
    The number of [records](entity:InventoryCount) to return.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .error import ErrorParams


class RevokeTokenResponseParams(typing_extensions.TypedDict):
    success: typing_extensions.NotRequired[bool]
    """
    If the request is successful, this is `true`.
    """

    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Any errors that occurred during the request.
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class CustomerSegment(UncheckedBaseModel):
    """
    Represents a group of customer profiles that match one or more predefined filter criteria.

    Segments (also known as Smart Groups) are defined and created within the Customer Directory in the
    Square Seller Dashboard or Point of Sale.
    """

    id: typing.Optional[str] = pydantic.Field(default=None)
    """
    A unique Square-generated ID for the segment.
    """

    name: str = pydantic.Field()
    """
    The name of the segment.
    """

    created_at: typing.Optional[str] = pydantic.Field(default=None)
    """
    The timestamp when the segment was created, in RFC 3339 format.
    """

    updated_at: typing.Optional[str] = pydantic.Field(default=None)
    """
    The timestamp when the segment was last updated, in RFC 3339 format.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.client_wrapper import SyncClientWrapper
import typing
from ..core.request_options import RequestOptions
from ..core.http_response import HttpResponse
from ..types.dismiss_terminal_action_response import DismissTerminalActionResponse
from ..core.jsonable_encoder import jsonable_encoder
from ..core.unchecked_base_model import construct_type
from json.decoder import JSONDecodeError
from ..core.api_error import ApiError
from ..types.dismiss_terminal_checkout_response import DismissTerminalCheckoutResponse
from ..types.dismiss_terminal_refund_response import DismissTerminalRefundResponse
from ..core.client_wrapper import AsyncClientWrapper
from ..core.http_response import AsyncHttpResponse


class RawTerminalClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def dismiss_terminal_action(
        self, action_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[DismissTerminalActionResponse]:
        """
        Dismisses a Terminal action request if the status and type of the request permits it.

        See [Link and Dismiss Actions](https://developer.squareup.com/docs/terminal-api/advanced-features/custom-workflows/link-and-dismiss-actions) for more details.

        Parameters
        ----------
        action_id : str
            Unique ID for the `TerminalAction` associated with the action to be dismissed.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[DismissTerminalActionResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/terminals/actions/{jsonable_encoder(action_id)}/dismiss",
            method="POST",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    DismissTerminalActionResponse,
                    construct_type(
                        type_=DismissTerminalActionResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def dismiss_terminal_checkout(
        self, checkout_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[DismissTerminalCheckoutResponse]:
        """
        Dismisses a Terminal checkout request if the status and type of the request permits it.

        Parameters
        ----------
        checkout_id : str
            Unique ID for the `TerminalCheckout` associated with the checkout to be dismissed.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[DismissTerminalCheckoutResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/terminals/checkouts/{jsonable_encoder(checkout_id)}/dismiss",
            method="POST",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    DismissTerminalCheckoutResponse,
                    construct_type(
                        type_=DismissTerminalCheckoutResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def dismiss_terminal_refund(
        self, terminal_refund_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[DismissTerminalRefundResponse]:
        """
        Dismisses a Terminal refund request if the status and type of the request permits it.

        Parameters
        ----------
        terminal_refund_id : str
            Unique ID for the `TerminalRefund` associated with the refund to be dismissed.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[DismissTerminalRefundResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/terminals/refunds/{jsonable_encoder(terminal_refund_id)}/dismiss",
            method="POST",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    DismissTerminalRefundResponse,
                    construct_type(
                        type_=DismissTerminalRefundResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncRawTerminalClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def dismiss_terminal_action(
        self, action_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[DismissTerminalActionResponse]:
        """
        Dismisses a Terminal action request if the status and type of the request permits it.

        See [Link and Dismiss Actions](https://developer.squareup.com/docs/terminal-api/advanced-features/custom-workflows/link-and-dismiss-actions) for more details.

        Parameters
        ----------
        action_id : str
            Unique ID for the `TerminalAction` associated with the action to be dismissed.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[DismissTerminalActionResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/terminals/actions/{jsonable_encoder(action_id)}/dismiss",
            method="POST",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    DismissTerminalActionResponse,
                    construct_type(
                        type_=DismissTerminalActionResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def dismiss_terminal_checkout(
        self, checkout_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[DismissTerminalCheckoutResponse]:
        """
        Dismisses a Terminal checkout request if the status and type of the request permits it.

        Parameters
        ----------
        checkout_id : str
            Unique ID for the `TerminalCheckout` associated with the checkout to be dismissed.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[DismissTerminalCheckoutResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/terminals/checkouts/{jsonable_encoder(checkout_id)}/dismiss",
            method="POST",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    DismissTerminalCheckoutResponse,
                    construct_type(
                        type_=DismissTerminalCheckoutResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def dismiss_terminal_refund(
        self, terminal_refund_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[DismissTerminalRefundResponse]:
        """
        Dismisses a Terminal refund request if the status and type of the request permits it.

        Parameters
        ----------
        terminal_refund_id : str
            Unique ID for the `TerminalRefund` associated with the refund to be dismissed.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[DismissTerminalRefundResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/terminals/refunds/{jsonable_encoder(terminal_refund_id)}/dismiss",
            method="POST",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    DismissTerminalRefundResponse,
                    construct_type(
                        type_=DismissTerminalRefundResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

BusinessAppointmentSettingsBookingLocationType = typing.Union[
    typing.Literal["BUSINESS_LOCATION", "CUSTOMER_LOCATION", "PHONE"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
from .date_range import DateRangeParams
from ..types.scheduled_shift_workday_matcher import ScheduledShiftWorkdayMatcher
import typing


class ScheduledShiftWorkdayParams(typing_extensions.TypedDict):
    """
    A `ScheduledShift` search query filter parameter that sets a range of days that
    a `Shift` must start or end in before passing the filter condition.
    """

    date_range: typing_extensions.NotRequired[DateRangeParams]
    """
    Dates for fetching the scheduled shifts.
    """

    match_scheduled_shifts_by: typing_extensions.NotRequired[ScheduledShiftWorkdayMatcher]
    """
    The strategy on which the dates are applied.
    See [ScheduledShiftWorkdayMatcher](#type-scheduledshiftworkdaymatcher) for possible values
    """

    default_timezone: typing_extensions.NotRequired[typing.Optional[str]]
    """
    Location-specific timezones convert workdays to datetime filters.
    Every location included in the query must have a timezone or this field
    must be provided as a fallback. Format: the IANA timezone database
    identifier for the relevant timezone.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing


class EventMetadataParams(typing_extensions.TypedDict):
    """
    Contains metadata about a particular [Event](entity:Event).
    """

    event_id: typing_extensions.NotRequired[typing.Optional[str]]
    """
    A unique ID for the event.
    """

    api_version: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The API version of the event. This corresponds to the default API version of the developer application at the time when the event was created.
    """



# This file was auto-generated by Fern from our API Definition.

from ...core.client_wrapper import SyncClientWrapper
from .raw_client import RawEvidenceClient
import typing
from ...core.request_options import RequestOptions
from ...core.pagination import SyncPager
from ...types.dispute_evidence import DisputeEvidence
from ...core.jsonable_encoder import jsonable_encoder
from ...types.list_dispute_evidence_response import ListDisputeEvidenceResponse
from ...core.unchecked_base_model import construct_type
from json.decoder import JSONDecodeError
from ...core.api_error import ApiError
from ...types.get_dispute_evidence_response import GetDisputeEvidenceResponse
from ...types.delete_dispute_evidence_response import DeleteDisputeEvidenceResponse
from ...core.client_wrapper import AsyncClientWrapper
from .raw_client import AsyncRawEvidenceClient
from ...core.pagination import AsyncPager


class EvidenceClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawEvidenceClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawEvidenceClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawEvidenceClient
        """
        return self._raw_client

    def list(
        self,
        dispute_id: str,
        *,
        cursor: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[DisputeEvidence]:
        """
        Returns a list of evidence associated with a dispute.

        Parameters
        ----------
        dispute_id : str
            The ID of the dispute.

        cursor : typing.Optional[str]
            A pagination cursor returned by a previous call to this endpoint.
            Provide this cursor to retrieve the next set of results for the original query.
            For more information, see [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[DisputeEvidence]
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        response = client.disputes.evidence.list(
            dispute_id="dispute_id",
        )
        for item in response:
            yield item
        # alternatively, you can paginate page-by-page
        for page in response.iter_pages():
            yield page
        """
        _response = self._raw_client._client_wrapper.httpx_client.request(
            f"v2/disputes/{jsonable_encoder(dispute_id)}/evidence",
            method="GET",
            params={
                "cursor": cursor,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    ListDisputeEvidenceResponse,
                    construct_type(
                        type_=ListDisputeEvidenceResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _parsed_next = _parsed_response.cursor
                _has_next = _parsed_next is not None and _parsed_next != ""
                _get_next = lambda: self.list(
                    dispute_id,
                    cursor=_parsed_next,
                    request_options=request_options,
                )
                _items = _parsed_response.evidence
                return SyncPager(has_next=_has_next, items=_items, get_next=_get_next)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get(
        self, dispute_id: str, evidence_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetDisputeEvidenceResponse:
        """
        Returns the metadata for the evidence specified in the request URL path.

        You must maintain a copy of any evidence uploaded if you want to reference it later. Evidence cannot be downloaded after you upload it.

        Parameters
        ----------
        dispute_id : str
            The ID of the dispute from which you want to retrieve evidence metadata.

        evidence_id : str
            The ID of the evidence to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetDisputeEvidenceResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.disputes.evidence.get(
            dispute_id="dispute_id",
            evidence_id="evidence_id",
        )
        """
        response = self._raw_client.get(dispute_id, evidence_id, request_options=request_options)
        return response.data

    def delete(
        self, dispute_id: str, evidence_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> DeleteDisputeEvidenceResponse:
        """
        Removes specified evidence from a dispute.
        Square does not send the bank any evidence that is removed.

        Parameters
        ----------
        dispute_id : str
            The ID of the dispute from which you want to remove evidence.

        evidence_id : str
            The ID of the evidence you want to remove.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DeleteDisputeEvidenceResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.disputes.evidence.delete(
            dispute_id="dispute_id",
            evidence_id="evidence_id",
        )
        """
        response = self._raw_client.delete(dispute_id, evidence_id, request_options=request_options)
        return response.data


class AsyncEvidenceClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawEvidenceClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawEvidenceClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawEvidenceClient
        """
        return self._raw_client

    async def list(
        self,
        dispute_id: str,
        *,
        cursor: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[DisputeEvidence]:
        """
        Returns a list of evidence associated with a dispute.

        Parameters
        ----------
        dispute_id : str
            The ID of the dispute.

        cursor : typing.Optional[str]
            A pagination cursor returned by a previous call to this endpoint.
            Provide this cursor to retrieve the next set of results for the original query.
            For more information, see [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[DisputeEvidence]
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            response = await client.disputes.evidence.list(
                dispute_id="dispute_id",
            )
            async for item in response:
                yield item
            # alternatively, you can paginate page-by-page
            async for page in response.iter_pages():
                yield page


        asyncio.run(main())
        """
        _response = await self._raw_client._client_wrapper.httpx_client.request(
            f"v2/disputes/{jsonable_encoder(dispute_id)}/evidence",
            method="GET",
            params={
                "cursor": cursor,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    ListDisputeEvidenceResponse,
                    construct_type(
                        type_=ListDisputeEvidenceResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _parsed_next = _parsed_response.cursor
                _has_next = _parsed_next is not None and _parsed_next != ""
                _get_next = lambda: self.list(
                    dispute_id,
                    cursor=_parsed_next,
                    request_options=request_options,
                )
                _items = _parsed_response.evidence
                return AsyncPager(has_next=_has_next, items=_items, get_next=_get_next)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get(
        self, dispute_id: str, evidence_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetDisputeEvidenceResponse:
        """
        Returns the metadata for the evidence specified in the request URL path.

        You must maintain a copy of any evidence uploaded if you want to reference it later. Evidence cannot be downloaded after you upload it.

        Parameters
        ----------
        dispute_id : str
            The ID of the dispute from which you want to retrieve evidence metadata.

        evidence_id : str
            The ID of the evidence to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetDisputeEvidenceResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.disputes.evidence.get(
                dispute_id="dispute_id",
                evidence_id="evidence_id",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.get(dispute_id, evidence_id, request_options=request_options)
        return response.data

    async def delete(
        self, dispute_id: str, evidence_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> DeleteDisputeEvidenceResponse:
        """
        Removes specified evidence from a dispute.
        Square does not send the bank any evidence that is removed.

        Parameters
        ----------
        dispute_id : str
            The ID of the dispute from which you want to remove evidence.

        evidence_id : str
            The ID of the evidence you want to remove.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DeleteDisputeEvidenceResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.disputes.evidence.delete(
                dispute_id="dispute_id",
                evidence_id="evidence_id",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.delete(dispute_id, evidence_id, request_options=request_options)
        return response.data



# This file was auto-generated by Fern from our API Definition.

import typing

CheckoutOptionsPaymentType = typing.Union[
    typing.Literal[
        "CARD_PRESENT",
        "MANUAL_CARD_ENTRY",
        "FELICA_ID",
        "FELICA_QUICPAY",
        "FELICA_TRANSPORTATION_GROUP",
        "FELICA_ALL",
        "PAYPAY",
        "QR_CODE",
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
from .loyalty_program_accrual_rule_type import LoyaltyProgramAccrualRuleType
import pydantic
import typing
from .loyalty_program_accrual_rule_visit_data import LoyaltyProgramAccrualRuleVisitData
from .loyalty_program_accrual_rule_spend_data import LoyaltyProgramAccrualRuleSpendData
from .loyalty_program_accrual_rule_item_variation_data import LoyaltyProgramAccrualRuleItemVariationData
from .loyalty_program_accrual_rule_category_data import LoyaltyProgramAccrualRuleCategoryData
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class LoyaltyProgramAccrualRule(UncheckedBaseModel):
    """
    Represents an accrual rule, which defines how buyers can earn points from the base [loyalty program](entity:LoyaltyProgram).
    """

    accrual_type: LoyaltyProgramAccrualRuleType = pydantic.Field()
    """
    The type of the accrual rule that defines how buyers can earn points.
    See [LoyaltyProgramAccrualRuleType](#type-loyaltyprogramaccrualruletype) for possible values
    """

    points: typing.Optional[int] = pydantic.Field(default=None)
    """
    The number of points that 
    buyers earn based on the `accrual_type`.
    """

    visit_data: typing.Optional[LoyaltyProgramAccrualRuleVisitData] = pydantic.Field(default=None)
    """
    Additional data for rules with the `VISIT` accrual type.
    """

    spend_data: typing.Optional[LoyaltyProgramAccrualRuleSpendData] = pydantic.Field(default=None)
    """
    Additional data for rules with the `SPEND` accrual type.
    """

    item_variation_data: typing.Optional[LoyaltyProgramAccrualRuleItemVariationData] = pydantic.Field(default=None)
    """
    Additional data for rules with the `ITEM_VARIATION` accrual type.
    """

    category_data: typing.Optional[LoyaltyProgramAccrualRuleCategoryData] = pydantic.Field(default=None)
    """
    Additional data for rules with the `CATEGORY` accrual type.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing


class CatalogCustomAttributeDefinitionSelectionConfigCustomAttributeSelectionParams(typing_extensions.TypedDict):
    """
    A named selection for this `SELECTION`-type custom attribute definition.
    """

    uid: typing_extensions.NotRequired[typing.Optional[str]]
    """
    Unique ID set by Square.
    """

    name: str
    """
    Selection name, unique within `allowed_selections`.
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
from .money import Money
import pydantic
import typing
from .gift_card_activity_redeem_status import GiftCardActivityRedeemStatus
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class GiftCardActivityRedeem(UncheckedBaseModel):
    """
    Represents details about a `REDEEM` [gift card activity type](entity:GiftCardActivityType).
    """

    amount_money: Money = pydantic.Field()
    """
    The amount deducted from the gift card for the redemption. This value is a positive integer.
    
    Applications that use a custom payment processing system must specify this amount in the 
    [CreateGiftCardActivity](api-endpoint:GiftCardActivities-CreateGiftCardActivity) request.
    """

    payment_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The ID of the payment that represents the gift card redemption. Square populates this field 
    if the payment was processed by Square.
    """

    reference_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    A client-specified ID that associates the gift card activity with an entity in another system. 
    
    Applications that use a custom payment processing system can use this field to track information
    related to an order or payment.
    """

    status: typing.Optional[GiftCardActivityRedeemStatus] = pydantic.Field(default=None)
    """
    The status of the gift card redemption. Gift cards redeemed from Square Point of Sale or the 
    Square Seller Dashboard use a two-state process: `PENDING` 
    to `COMPLETED` or `PENDING` to  `CANCELED`. Gift cards redeemed using the Gift Card Activities API 
    always have a `COMPLETED` status.
    See [Status](#type-status) for possible values
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
from ...core.client_wrapper import SyncClientWrapper
from ...requests.shift import ShiftParams
from ...core.request_options import RequestOptions
from ...core.http_response import HttpResponse
from ...types.create_shift_response import CreateShiftResponse
from ...core.serialization import convert_and_respect_annotation_metadata
from ...core.unchecked_base_model import construct_type
from json.decoder import JSONDecodeError
from ...core.api_error import ApiError
from ...requests.shift_query import ShiftQueryParams
from ...types.search_shifts_response import SearchShiftsResponse
from ...types.get_shift_response import GetShiftResponse
from ...core.jsonable_encoder import jsonable_encoder
from ...types.update_shift_response import UpdateShiftResponse
from ...types.delete_shift_response import DeleteShiftResponse
from ...core.client_wrapper import AsyncClientWrapper
from ...core.http_response import AsyncHttpResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawShiftsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def create(
        self,
        *,
        shift: ShiftParams,
        idempotency_key: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[CreateShiftResponse]:
        """
        Creates a new `Shift`.

        A `Shift` represents a complete workday for a single team member.
        You must provide the following values in your request to this
        endpoint:

        - `location_id`
        - `team_member_id`
        - `start_at`

        An attempt to create a new `Shift` can result in a `BAD_REQUEST` error when:
        - The `status` of the new `Shift` is `OPEN` and the team member has another
        shift with an `OPEN` status.
        - The `start_at` date is in the future.
        - The `start_at` or `end_at` date overlaps another shift for the same team member.
        - The `Break` instances are set in the request and a break `start_at`
        is before the `Shift.start_at`, a break `end_at` is after
        the `Shift.end_at`, or both.

        Parameters
        ----------
        shift : ShiftParams
            The `Shift` to be created.

        idempotency_key : typing.Optional[str]
            A unique string value to ensure the idempotency of the operation.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[CreateShiftResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            "v2/labor/shifts",
            method="POST",
            json={
                "idempotency_key": idempotency_key,
                "shift": convert_and_respect_annotation_metadata(
                    object_=shift, annotation=ShiftParams, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreateShiftResponse,
                    construct_type(
                        type_=CreateShiftResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def search(
        self,
        *,
        query: typing.Optional[ShiftQueryParams] = OMIT,
        limit: typing.Optional[int] = OMIT,
        cursor: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[SearchShiftsResponse]:
        """
        Returns a paginated list of `Shift` records for a business.
        The list to be returned can be filtered by:
        - Location IDs
        - Team member IDs
        - Shift status (`OPEN` or `CLOSED`)
        - Shift start
        - Shift end
        - Workday details

        The list can be sorted by:
        - `START_AT`
        - `END_AT`
        - `CREATED_AT`
        - `UPDATED_AT`

        Parameters
        ----------
        query : typing.Optional[ShiftQueryParams]
            Query filters.

        limit : typing.Optional[int]
            The number of resources in a page (200 by default).

        cursor : typing.Optional[str]
            An opaque cursor for fetching the next page.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[SearchShiftsResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            "v2/labor/shifts/search",
            method="POST",
            json={
                "query": convert_and_respect_annotation_metadata(
                    object_=query, annotation=ShiftQueryParams, direction="write"
                ),
                "limit": limit,
                "cursor": cursor,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    SearchShiftsResponse,
                    construct_type(
                        type_=SearchShiftsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[GetShiftResponse]:
        """
        Returns a single `Shift` specified by `id`.

        Parameters
        ----------
        id : str
            The UUID for the `Shift` being retrieved.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[GetShiftResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/labor/shifts/{jsonable_encoder(id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetShiftResponse,
                    construct_type(
                        type_=GetShiftResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def update(
        self, id: str, *, shift: ShiftParams, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[UpdateShiftResponse]:
        """
        Updates an existing `Shift`.

        When adding a `Break` to a `Shift`, any earlier `Break` instances in the `Shift` have
        the `end_at` property set to a valid RFC-3339 datetime string.

        When closing a `Shift`, all `Break` instances in the `Shift` must be complete with `end_at`
        set on each `Break`.

        Parameters
        ----------
        id : str
            The ID of the object being updated.

        shift : ShiftParams
            The updated `Shift` object.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[UpdateShiftResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/labor/shifts/{jsonable_encoder(id)}",
            method="PUT",
            json={
                "shift": convert_and_respect_annotation_metadata(
                    object_=shift, annotation=ShiftParams, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpdateShiftResponse,
                    construct_type(
                        type_=UpdateShiftResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def delete(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[DeleteShiftResponse]:
        """
        Deletes a `Shift`.

        Parameters
        ----------
        id : str
            The UUID for the `Shift` being deleted.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[DeleteShiftResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/labor/shifts/{jsonable_encoder(id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    DeleteShiftResponse,
                    construct_type(
                        type_=DeleteShiftResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncRawShiftsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def create(
        self,
        *,
        shift: ShiftParams,
        idempotency_key: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[CreateShiftResponse]:
        """
        Creates a new `Shift`.

        A `Shift` represents a complete workday for a single team member.
        You must provide the following values in your request to this
        endpoint:

        - `location_id`
        - `team_member_id`
        - `start_at`

        An attempt to create a new `Shift` can result in a `BAD_REQUEST` error when:
        - The `status` of the new `Shift` is `OPEN` and the team member has another
        shift with an `OPEN` status.
        - The `start_at` date is in the future.
        - The `start_at` or `end_at` date overlaps another shift for the same team member.
        - The `Break` instances are set in the request and a break `start_at`
        is before the `Shift.start_at`, a break `end_at` is after
        the `Shift.end_at`, or both.

        Parameters
        ----------
        shift : ShiftParams
            The `Shift` to be created.

        idempotency_key : typing.Optional[str]
            A unique string value to ensure the idempotency of the operation.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[CreateShiftResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v2/labor/shifts",
            method="POST",
            json={
                "idempotency_key": idempotency_key,
                "shift": convert_and_respect_annotation_metadata(
                    object_=shift, annotation=ShiftParams, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreateShiftResponse,
                    construct_type(
                        type_=CreateShiftResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def search(
        self,
        *,
        query: typing.Optional[ShiftQueryParams] = OMIT,
        limit: typing.Optional[int] = OMIT,
        cursor: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[SearchShiftsResponse]:
        """
        Returns a paginated list of `Shift` records for a business.
        The list to be returned can be filtered by:
        - Location IDs
        - Team member IDs
        - Shift status (`OPEN` or `CLOSED`)
        - Shift start
        - Shift end
        - Workday details

        The list can be sorted by:
        - `START_AT`
        - `END_AT`
        - `CREATED_AT`
        - `UPDATED_AT`

        Parameters
        ----------
        query : typing.Optional[ShiftQueryParams]
            Query filters.

        limit : typing.Optional[int]
            The number of resources in a page (200 by default).

        cursor : typing.Optional[str]
            An opaque cursor for fetching the next page.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[SearchShiftsResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v2/labor/shifts/search",
            method="POST",
            json={
                "query": convert_and_respect_annotation_metadata(
                    object_=query, annotation=ShiftQueryParams, direction="write"
                ),
                "limit": limit,
                "cursor": cursor,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    SearchShiftsResponse,
                    construct_type(
                        type_=SearchShiftsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[GetShiftResponse]:
        """
        Returns a single `Shift` specified by `id`.

        Parameters
        ----------
        id : str
            The UUID for the `Shift` being retrieved.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[GetShiftResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/labor/shifts/{jsonable_encoder(id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetShiftResponse,
                    construct_type(
                        type_=GetShiftResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def update(
        self, id: str, *, shift: ShiftParams, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[UpdateShiftResponse]:
        """
        Updates an existing `Shift`.

        When adding a `Break` to a `Shift`, any earlier `Break` instances in the `Shift` have
        the `end_at` property set to a valid RFC-3339 datetime string.

        When closing a `Shift`, all `Break` instances in the `Shift` must be complete with `end_at`
        set on each `Break`.

        Parameters
        ----------
        id : str
            The ID of the object being updated.

        shift : ShiftParams
            The updated `Shift` object.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[UpdateShiftResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/labor/shifts/{jsonable_encoder(id)}",
            method="PUT",
            json={
                "shift": convert_and_respect_annotation_metadata(
                    object_=shift, annotation=ShiftParams, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpdateShiftResponse,
                    construct_type(
                        type_=UpdateShiftResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def delete(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[DeleteShiftResponse]:
        """
        Deletes a `Shift`.

        Parameters
        ----------
        id : str
            The UUID for the `Shift` being deleted.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[DeleteShiftResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/labor/shifts/{jsonable_encoder(id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    DeleteShiftResponse,
                    construct_type(
                        type_=DeleteShiftResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
from .tender_bank_account_details_status import TenderBankAccountDetailsStatus
import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class TenderBankAccountDetails(UncheckedBaseModel):
    """
    Represents the details of a tender with `type` `BANK_ACCOUNT`.

    See [BankAccountPaymentDetails](entity:BankAccountPaymentDetails)
    for more exposed details of a bank account payment.
    """

    status: typing.Optional[TenderBankAccountDetailsStatus] = pydantic.Field(default=None)
    """
    The bank account payment's current state.
    
    See [TenderBankAccountPaymentDetailsStatus](entity:TenderBankAccountDetailsStatus) for possible values.
    See [TenderBankAccountDetailsStatus](#type-tenderbankaccountdetailsstatus) for possible values
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions


class LoyaltyEventRedeemRewardParams(typing_extensions.TypedDict):
    """
    Provides metadata when the event `type` is `REDEEM_REWARD`.
    """

    loyalty_program_id: str
    """
    The ID of the [loyalty program](entity:LoyaltyProgram).
    """

    reward_id: typing_extensions.NotRequired[str]
    """
    The ID of the redeemed [loyalty reward](entity:LoyaltyReward).
    This field is returned only if the event source is `LOYALTY_API`.
    """

    order_id: typing_extensions.NotRequired[str]
    """
    The ID of the [order](entity:Order) that redeemed the reward.
    This field is returned only if the Orders API is used to process orders.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing


class CoordinatesParams(typing_extensions.TypedDict):
    """
    Latitude and longitude coordinates.
    """

    latitude: typing_extensions.NotRequired[typing.Optional[float]]
    """
    The latitude of the coordinate expressed in degrees.
    """

    longitude: typing_extensions.NotRequired[typing.Optional[float]]
    """
    The longitude of the coordinate expressed in degrees.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing


class CustomerDetailsParams(typing_extensions.TypedDict):
    """
    Details about the customer making the payment.
    """

    customer_initiated: typing_extensions.NotRequired[typing.Optional[bool]]
    """
    Indicates whether the customer initiated the payment.
    """

    seller_keyed_in: typing_extensions.NotRequired[typing.Optional[bool]]
    """
    Indicates that the seller keyed in payment details on behalf of the customer.
    This is used to flag a payment as Mail Order / Telephone Order (MOTO).
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
from .measurement_unit import MeasurementUnit
import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class OrderQuantityUnit(UncheckedBaseModel):
    """
    Contains the measurement unit for a quantity and a precision that
    specifies the number of digits after the decimal point for decimal quantities.
    """

    measurement_unit: typing.Optional[MeasurementUnit] = pydantic.Field(default=None)
    """
    A [MeasurementUnit](entity:MeasurementUnit) that represents the
    unit of measure for the quantity.
    """

    precision: typing.Optional[int] = pydantic.Field(default=None)
    """
    For non-integer quantities, represents the number of digits after the decimal point that are
    recorded for this quantity.
    
    For example, a precision of 1 allows quantities such as `"1.0"` and `"1.1"`, but not `"1.01"`.
    
    Min: 0. Max: 5.
    """

    catalog_object_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The catalog object ID referencing the
    [CatalogMeasurementUnit](entity:CatalogMeasurementUnit).
    
    This field is set when this is a catalog-backed measurement unit.
    """

    catalog_version: typing.Optional[int] = pydantic.Field(default=None)
    """
    The version of the catalog object that this measurement unit references.
    
    This field is set when this is a catalog-backed measurement unit.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
from .error import Error
import pydantic
from .customer import Customer
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class ListCustomersResponse(UncheckedBaseModel):
    """
    Defines the fields that are included in the response body of
    a request to the `ListCustomers` endpoint.

    Either `errors` or `customers` is present in a given response (never both).
    """

    errors: typing.Optional[typing.List[Error]] = pydantic.Field(default=None)
    """
    Any errors that occurred during the request.
    """

    customers: typing.Optional[typing.List[Customer]] = pydantic.Field(default=None)
    """
    The customer profiles associated with the Square account or an empty object (`{}`) if none are found.
    Only customer profiles with public information (`given_name`, `family_name`, `company_name`, `email_address`, or
    `phone_number`) are included in the response.
    """

    cursor: typing.Optional[str] = pydantic.Field(default=None)
    """
    A pagination cursor to retrieve the next set of results for the
    original query. A cursor is only present if the request succeeded and additional results
    are available.
    
    For more information, see [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination).
    """

    count: typing.Optional[int] = pydantic.Field(default=None)
    """
    The total count of customers associated with the Square account. Only customer profiles with public information
    (`given_name`, `family_name`, `company_name`, `email_address`, or `phone_number`) are counted. This field is present
    only if `count` is set to `true` in the request.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
from .customer_creation_source import CustomerCreationSource
import pydantic
from .customer_inclusion_exclusion import CustomerInclusionExclusion
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class CustomerCreationSourceFilter(UncheckedBaseModel):
    """
    The creation source filter.

    If one or more creation sources are set, customer profiles are included in,
    or excluded from, the result if they match at least one of the filter criteria.
    """

    values: typing.Optional[typing.List[CustomerCreationSource]] = pydantic.Field(default=None)
    """
    The list of creation sources used as filtering criteria.
    See [CustomerCreationSource](#type-customercreationsource) for possible values
    """

    rule: typing.Optional[CustomerInclusionExclusion] = pydantic.Field(default=None)
    """
    Indicates whether a customer profile matching the filter criteria
    should be included in the result or excluded from the result.
    
    Default: `INCLUDE`.
    See [CustomerInclusionExclusion](#type-customerinclusionexclusion) for possible values
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
